app.controller('AuthCtrl', function ($scope, $stateParams, $http, $state, localStorageService, $ionicModal, $ionicPopup, $rootScope,
                                     $ionicLoading, $ionicHistory, UserService, TimesheetService, $q, $localStorage) {
    // Check to see if user is already logged in
    var timeout = 30000;
    var idToken = $localStorage.token;
    $scope.user = {};

    if ($localStorage.user || idToken) {
        console.dir('logged in already, going to profile');
        $state.go('app.timesheet');
        return;
    }

    // Try to remove modal backdrops. - issue with modal not being destroyed and then not opening after logout
    $('.modal-backdrop').remove();
    $ionicModal.fromTemplateUrl('templates/auth/register.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (registerModal) {
        $scope.registerModal = registerModal;
    });

    $ionicModal.fromTemplateUrl('templates/auth/login.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (loginModal) {
        $scope.loginModal = loginModal;
    });

    $scope.openRegisterModal = function () {
        $scope.registerModal.show();
        $scope.user.email = null;
        $scope.user.password = null;
    }
    $scope.openLoginModal = function () {
        console.dir('load modal');
        console.dir($scope.loginModal);
        $scope.loginModal.show();
        $scope.user.email = null;
        $scope.user.password = null;
    }

    $scope.closeModal = function () {
        console.dir('close modal');
        $scope.loginModal.hide();
        $scope.registerModal.hide();
        $scope.user.email = null;
        $scope.user.password = null;
    };

    $scope.signIn = function (form) {

        if (!form.$valid) {
            return false;
        }

        $ionicLoading.show({
            template: '<ion-spinner class="light"></ion-spinner><br />Signing in...',
            content: '',
        });

        var data = {
            username: $scope.user.email,
            password: $scope.user.password,
            grant_type: 'password'
        };

        login(data);

    };


    $scope.register = function (form) {

        console.dir('register');
        if (!form.$valid) {
            return false;
        }

        $ionicLoading.show({
            template: '<ion-spinner class="light"></ion-spinner><br />Registering...',
            content: '',
        });

        var data = {
            name: $scope.user.name,
            email: $scope.user.email,
            plainPassword: $scope.user.password
        };

        var url = baseURL + 'authorize/register';

        $http({
            method: 'post',
            url: url,
            data: data,
            timeout: timeout,

        }).success(function (data) {
            console.dir(data);

            $ionicLoading.hide();
            if (data && data != null && data.errors) {

                // TODO - gross
                var errors = data.errors.replace("\n", "<br />");
                console.dir(errors);
                $('#form-errors').html(errors);
            } else if (!data) {
                $('#form-errors').html('Could not connect to the server');
            } else {
                console.dir('success');

                $localStorage.user = data['user'];
                $localStorage.token = data['token']['access_token'];
                $localStorage.refreshToken = data['token']['refresh_token']
                localStorageService.set('authorizationData', {token: data['token']['access_token']});

                UserService.setAll(data);
                TimesheetService.setTimesheets(data['timesheets']);

                console.dir('resetting views');
                $ionicHistory.clearCache();
                saveUser();

                $state.go('welcome.intro');

                $scope.registerModal.hide();
            }


        }).error(function (data, status, headers, config) {
            $ionicLoading.hide();
            console.dir('reg error');
            console.dir(data);
            console.dir(data.errors);
            console.dir($('#status-register'));

            if (data.errors)
                $('#status-register').text(data.errors).slideDown();
            else
                $('#status-register').html('Could not connect to the server').slideDown();


        });
    };

    var fbLoginSuccess = function (response) {
        if (!response.authResponse) {
            fbLoginError("Cannot find the authResponse");
            return;
        }

        console.dir('response');

        var authResponse = response.authResponse;

        getFacebookProfileInfo(authResponse)
            .then(function (profileInfo) {
                console.dir('fb profile');
                console.dir(profileInfo);

                var data = {
                    'username': authResponse.userID,
                    'password': authResponse.accessToken,
                    'grant_type': 'password'
                };

                console.dir(data);
                login(data, 'facebook');

            }, function (fail) {
                // Fail get profile info
                console.log('profile info fail', fail);
            });
    };

    var fbLoginError = function (error) {
        console.log('fbLoginError', error);

        $ionicPopup.alert({
            title: 'Error',
            template: error
        });

        $ionicLoading.hide();
    };

    // This method is to get the user profile info from the facebook api
    var getFacebookProfileInfo = function (authResponse) {
        var info = $q.defer();

        facebookConnectPlugin.api('/me?fields=email,name&access_token=' + authResponse.accessToken, null,
            function (response) {
                console.log(response);
                info.resolve(response);


            },
            function (response) {
                console.log(response);
                info.reject(response);
            }
        );
        return info.promise;
    };


    $scope.facebookSignIn = function () {

        facebookConnectPlugin.logout(function (status) {
                console.dir('fb logout');
                console.dir(status);
            },
            function (fail) {
                console.dir('fail logout');
                console.dir(fail);
            });

        facebookConnectPlugin.getLoginStatus(function (success) {
            console.dir('facebook');
            console.dir(success);

            // Should never hit this since we are forcing logout?
            // TODO
            if (success.status === 'connected') {
                console.log('getLoginStatus', success.status);

                getFacebookProfileInfo(success.authResponse).then(function (profileInfo) {
                    console.dir('facebook success - already logged in');
                    console.dir(profileInfo);
                }, function (fail) {
                    // Fail get profile info
                    console.log('profile info fail', fail);
                });

            } else {
                console.log('getLoginStatus', success.status);
                facebookConnectPlugin.login(['email', 'public_profile'], fbLoginSuccess, fbLoginError);
            }
        });
    };


    $scope.googleSignIn = function () {
        console.dir('google sign in');

        window.plugins.googleplus.login(
            {
                // 'scopes': 'profile',
                'webClientId' : '649052459608-ntvlpk5dlj9lfo6hiv1apsui7lnlos2e.apps.googleusercontent.com',
                'offline': true,
            },
            function (authResponse) {

                $ionicLoading.show({
                    template: 'Logging in...'
                });

                if (authResponse.serverAuthCode) {
                    var username = 'android';
                    var password = authResponse.serverAuthCode
                } else {
                    username = authResponse.idToken;
                    password = authResponse.accessToken;
                }

                if (authResponse.idToken) {
                    console.dir('google 1');
                    var data = {
                        'username': username,
                        'password': password,
                        'grant_type': 'password'
                    };
                }

                login(data, 'google');

            },
            function (msg) {
                console.dir('fail');
                console.dir(msg);

                // $ionicPopup.alert({
                //     title: 'Error',
                //     template: msg
                // });
                $ionicLoading.hide();
            }
        );
    };


    var login = function (data, network) {
        network = (typeof network === 'undefined') ? '' : network;

        if (!network) {
            var url = baseURL + 'authorize/login';
        } else {
            var url = baseURL + 'authorize/login/' + network;
        }

        $ionicLoading.show({
            template: '<ion-spinner class="light"></ion-spinner><br />Logging in...',
            content: '',
        });

        $http({
            method: 'post',
            url: url,
            data: data,
            timeout: timeout,
        }).success(function (data) {
            console.dir('login success: ');
            console.dir(data);
            $ionicLoading.hide();
            $ionicHistory.clearCache();
            $scope.user = data['user'];
            saveUser();

            if (data.error) {
                console.dir('ERROR: ');
                $('#status-signin').slideDown().text(data.error_description);
                return
            }

            $localStorage.user = data['user'];
            $localStorage.token = data['token'].access_token;
            $localStorage.refreshToken = data['token'].refresh_token;

            localStorageService.set('authorizationData', {token: data['token'].access_token});
            UserService.setAll(data);
            TimesheetService.setTimesheets(data['timesheets']);

            $scope.user = data['user'];

            // Has at least 1 company/client or employer
            if (data['data']['companies'].length || data['data']['clients'].length || data['user']['employers'].length) {
                $state.go('app.timesheet', {}, {reload: true});
            } else {
                $state.go('welcome.intro', {}, {reload: true});
            }
            console.dir('close modal');
            $scope.loginModal.hide();

        }).error(function (data, status, headers, config) {
            console.dir('login error');
            console.dir(data);

            if (data && data.error)
                var error = data.error;
            else
                var error = 'Something went wrong :(';

            $('#status-signin').html(error).slideDown();
            $ionicLoading.hide();

        });
    }


    var saveUser = function () {

        // user didnt set properly
        if (!$scope.user) {
            $ionicHistory.clearCache();
            $ionicHistory.clearHistory();

            $localStorage.$reset();

            localStorageService.clearAll();
            window.localStorage.clear();


            $ionicHistory.nextViewOptions({
                historyRoot: true,
                disableBack: true,
            });
            $scope.user = null;

            // $scope.clearTimers();

            // $timeout(function () {
                $ionicHistory.clearCache();
                $ionicHistory.clearHistory();
                $state.go('account', {}, {reload: true});
            // });
        }

        var attempted = false;

        // Not on a device
        if (typeof window.analytics === 'undefined') {
            return;
        }

        window.analytics.setUserId($scope.user.email);

    }

});


app.factory('Utils', function ($q) {
    return {
        isImage: function (src) {


            var deferred = $q.defer();

            var image = new Image();
            image.onerror = function () {
                deferred.resolve(false);
            };
            image.onload = function () {
                deferred.resolve(true);
            };
            image.src = src;

            return deferred.promise;

        }
    };
});
